/*
 * Copyright (c) 2024. Das Land Schleswig-Holstein vertreten durch das Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.common.errorhandling;

import java.util.Map;

import jakarta.validation.ConstraintViolationException;

import org.springframework.data.rest.webmvc.ResourceNotFoundException;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.ErrorResponse;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import de.ozgcloud.common.errorhandling.TechnicalException;

@RestControllerAdvice
public class ExceptionController extends ResponseEntityExceptionHandler {

	static final Map<Class<? extends Exception>, HttpStatus> STATUS_BY_EXCEPTION = Map.of(
			RuntimeException.class, HttpStatus.INTERNAL_SERVER_ERROR,
			AccessDeniedException.class, HttpStatus.FORBIDDEN,
			ConstraintViolationException.class, HttpStatus.UNPROCESSABLE_ENTITY,
			ResourceNotFoundException.class, HttpStatus.NOT_FOUND,
			FunctionalException.class, HttpStatus.BAD_REQUEST,
			TechnicalException.class, HttpStatus.INTERNAL_SERVER_ERROR);

	@ExceptionHandler({
			RuntimeException.class,
			AccessDeniedException.class,
			ConstraintViolationException.class,
			ResourceNotFoundException.class,
			FunctionalException.class,
			TechnicalException.class,
	})
	public ErrorResponse handleException(RuntimeException ex) {
		var status = STATUS_BY_EXCEPTION.getOrDefault(ex.getClass(), HttpStatus.INTERNAL_SERVER_ERROR);

		return ErrorResponse.builder(ex, status, ex.getLocalizedMessage()).build();
	}

}
