/*
 * Copyright (c) 2024. Das Land Schleswig-Holstein vertreten durch das Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin;

import static de.ozgcloud.admin.RootModelAssembler.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.boot.autoconfigure.data.rest.RepositoryRestProperties;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;

@ExtendWith(MockitoExtension.class)
class RootModelAssemblerTest {

	private static final String BASE_PATH = "/api/base";
	@Spy
	@InjectMocks
	private RootModelAssembler modelAssembler;

	@Mock
	private RepositoryRestProperties restProperties;

	@BeforeEach
	void mockBasePath() {
		when(restProperties.getBasePath()).thenReturn(BASE_PATH);
	}

	@DisplayName("Entity Model")
	@Nested
	class TestEntityModel {

		@Test
		void shouldHaveHrefToBasePath() {
			var configurationLink = toModel().getLink(REL_CONFIGURATION);

			assertEquals(Optional.of(Link.of(BASE_PATH, REL_CONFIGURATION)), configurationLink);
		}

		@Test
		void shouldHaveHrefToSelf() {
			var selfLink = toModel().getLink(IanaLinkRelations.SELF);

			assertEquals(Optional.of(Link.of(RootController.PATH)), selfLink);
		}

		private EntityModel<Root> toModel() {
			return modelAssembler.toModel(RootTestFactory.create());
		}

	}

}