/*
 * Copyright (c) 2024. Das Land Schleswig-Holstein vertreten durch das Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.common.errorhandling;

import java.util.Collections;
import java.util.Map;

import jakarta.validation.ConstraintViolationException;

import org.springframework.data.rest.webmvc.ResourceNotFoundException;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import de.ozgcloud.common.errorhandling.TechnicalException;

@RestController
@RequestMapping("/api/test-error")
class TestErrorController {

	@FunctionalInterface
	interface ExceptionProducer {
		Exception produceException();
	}

	static final String ERROR_MESSAGE = "error message";

	static Map<Class<? extends Exception>, ExceptionProducer> EXCEPTION_PRODUCER = Map.of(
			RuntimeException.class, () -> new RuntimeException(ERROR_MESSAGE),
			AccessDeniedException.class, () -> new AccessDeniedException(ERROR_MESSAGE),
			ConstraintViolationException.class, () -> new ConstraintViolationException(ERROR_MESSAGE, Collections.emptySet()),
			ResourceNotFoundException.class, () -> new ResourceNotFoundException(ERROR_MESSAGE),
			FunctionalException.class, () -> new FunctionalException(() -> ERROR_MESSAGE),
			TechnicalException.class, () -> new TechnicalException(ERROR_MESSAGE));

	@GetMapping
	String throwException(@RequestParam String errorClassName) throws Exception {
		throw EXCEPTION_PRODUCER.get(
				Class.forName(errorClassName)).produceException();
	}

}
