package de.ozgcloud.admin.configurationparameter;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.config.environment.Environment;
import org.springframework.cloud.config.server.environment.EnvironmentRepository;
import org.springframework.security.test.context.support.WithMockUser;

import de.ozgcloud.common.test.DataITCase;

@DataITCase
@WithMockUser
class AdminEnvironmentRepositoryITCase {

	@Autowired
	private EnvironmentRepository repository;
	@Autowired
	private PropertyRepository propertyRepository;

	private ConfigurationProperties configurationProperties = ConfigurationPropertiesTestFactory.create();

	@AfterEach
	void clearData() {
		propertyRepository.deleteAll();
	}

	@Nested
	class TestFindOne {
		@BeforeEach
		void initDb() {
			saveMongoSource(configurationProperties);
		}

		@Test
		void shouldFindEnvironmentWithOneCorrectPropertySource() {

			var environment = findOneEnvironment();

			assertThat(environment.getPropertySources()).hasSize(1);
		}

		@Test
		void shouldFindEnvironmentWithTwoCorrectPropertySources() {
			var configurationProperties2 = ConfigurationPropertiesTestFactory.createBuilder().profile("other").build();
			saveMongoSource(configurationProperties2);

			var environment = findOneEnvironment();

			assertThat(environment.getPropertySources()).hasSize(2);
		}

		private void saveMongoSource(ConfigurationProperties configurationProperties) {
			propertyRepository.save(configurationProperties);
		}

		@Test
		void shouldFindEnvironmentWithoutPropertySources() {

			var environment = repository.findOne("FalseAppName", ConfigurationPropertiesTestFactory.PROFILE, null);

			assertThat(environment.getPropertySources()).isEmpty();

		}

		@Test
		void shouldFindCorrectEnvironment() {

			var environment = findOneEnvironment();

			assertThat(environment.getPropertySources().getLast().getSource()).isEqualTo(ConfigurationPropertiesTestFactory.PROPERTIES);
		}

		private Environment findOneEnvironment() {
			return repository.findOne(ConfigurationPropertiesTestFactory.APPNAME, ConfigurationPropertiesTestFactory.PROFILE,
					null);
		}

	}

}
